package connection;

import currency.*;
import java.io.DataInputStream;
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;

/**
 * Gets currency information across a given API.
 * (API provides an XML, InternetConnection parses that XML)
 */
public class InternetConnection {

    private String url;
    private Currency currency;

    /**
    * Constructor
    */
    public InternetConnection(String bank, String currency) throws CurrencyException, InternetConnectionException {
        if (!Banks.isValidBank(bank) || !Currencies.isValidCurrency(currency)) {
            throw new CurrencyException("Bank or currency is invalid!");
        }

        this.currency = new Currency(bank, currency);

        url = "http://api.napiarfolyam.hu/?bank=" + bank + "&valuta=" + currency;

        HttpConnection server;

        try {
            server = ((HttpConnection) Connector.open(url, Connector.READ, true));

            if (server.getResponseCode() != HttpConnection.HTTP_OK) {
                throw new InternetConnectionException("HTTP error");
            }

            readCurrencyInformation(server.openDataInputStream());

        } catch (IOException ex) {
            throw new InternetConnectionException("Error with connection: " + ex.getMessage());
        }
    }

    /**
    * Gets data from internet using HTTP protocol.
    */
    private void readCurrencyInformation(DataInputStream input) throws IOException {
        String tag = "";

        do {
            tag = getNextTag(input);

            try {
                if ("vetel".equals(tag)) {
                    currency.setBuy(Float.parseFloat(getData(input)));
                } else if ("elad".equals(tag.substring(0, 4))) {
                    currency.setSale(Float.parseFloat(getData(input)));
                }
            } catch (Exception ex) {
                throw new IOException("Wrong XML file format: " + tag);
            }

        } while (!currency.isSet() && (input.available() > 0));
    }

    /**
    * Finds next parsable XML-tag of become data.
    */
    private String getNextTag(DataInputStream input) throws IOException {
        char[] actualRead = new char[1];

        do {
            actualRead[0] = (char) input.readByte();
        } while (actualRead[0] != '<');

        byte[] tag = new byte[128];

        int counter = -1;
        char last = ' ';
        do {
            counter++;
            tag[counter] = input.readByte();
            last = (char)tag[counter];
        } while ((char)tag[counter] != '>');

        byte[] temp = new byte[counter];
        for (int i = 0; i < counter; i++) {
            temp[i] = tag[i];
        }

        return new String(temp);
    }

    /**
    * Collects data from the internet connection.
    */
    private String getData(DataInputStream input) throws IOException {
        byte[] tag = new byte[128];
        int counter = -1;
        do {
            counter++;
            tag[counter] = input.readByte();
        } while ((char)tag[counter] != '<');

        byte[] temp = new byte[counter];
        for (int i = 0; i < counter; i++) {
            temp[i] = tag[i];
        }

        return new String(temp);
    }

    public Currency getCurrency() {
        return new Currency(currency);
    }

    public String getCurrencyWritten() {
        return currency.toString();
    }
}
